<?php

namespace Moneta;

use Moneta\MonetaSdkService;
use Moneta\MonetaSdkSettings;
use Moneta\MonetaSdkReceipt;

class MonetaSdk {
    /**
     * Assistant endpoints
     */
    const PROD_ENDPOINT = 'https://www.payanyway.ru/assistant.htm?';
    const DEMO_ENDPOINT = 'https://demo.moneta.ru/assistant.htm?';

    /**
     * Exception messages text
     */
    const EXCEPTION_MERCHANT_API_NO_TRANSACTION_ID = 'There is no operation number in the response.';
    const EXCEPTION_VAR_AMOUNT_EMPTY = 'The amount argument is not passed.';

    private
        $configPath,
        $transactionId,
        $amount,
        $description;

    public function __construct($transactionId, ?float $amount = null, ?string $description = null, ?string $configPath = null) {
        $this->configPath = $configPath;
        $this->transactionId = $transactionId;
        $this->amount = $amount;
        $this->description = $description;
    }

    /**
     * @return string
     * @throws MonetaSdkException
     */
    public function getAssistantPaymentLink() {
        if (!isset($this->amount)) {
            throw new MonetaSdkException(self::EXCEPTION_VAR_AMOUNT_EMPTY);
        }

        $httpParams = [
            'MNT_ID' => MonetaSdkSettings::getSettings('id', $this->configPath),
            'MNT_AMOUNT' => number_format($this->amount, 2, '.', ''),
            'MNT_TRANSACTION_ID' => $this->transactionId,
            'MNT_CURRENCY_CODE' => 'RUB',
            'MNT_DESCRIPTION' => $this->description,
            'MNT_SIGNATURE' => MonetaSdkService::createSignature('generateLink', $this->transactionId, $this->configPath, $this->amount),
            'MNT_SUCCESS_URL' => MonetaSdkSettings::getSettings('success', $this->configPath),
            'MNT_INPROGRESS_URL' => MonetaSdkSettings::getSettings('inprogress', $this->configPath),
            'MNT_FAIL_URL' => MonetaSdkSettings::getSettings('fail', $this->configPath),
            'MNT_RETURN_URL' => MonetaSdkSettings::getSettings('return', $this->configPath),
            'moneta.locale' => MonetaSdkSettings::getSettings('locale', $this->configPath),
            'paymentSystem.unitId' => MonetaSdkSettings::getSettings('unit_id', $this->configPath),
            'paymentSystem.limitIds' => MonetaSdkSettings::getSettings('limit_ids', $this->configPath),
        ];

        $httpParams = array_diff($httpParams, ['']);

        $endpoint = MonetaSdkSettings::getSettings('demo_mode', $this->configPath) ? self::DEMO_ENDPOINT : self::PROD_ENDPOINT;

        $paymentLink = $endpoint . http_build_query($httpParams);

        return $paymentLink;
    }

    /**
     * @param MonetaSdkReceipt $mSdkR
     * @param string $transactionId
     * @return string
     * @throws MonetaSdkException
     */
    public function responseToCheckRequest(MonetaSdkReceipt $mSdkR) {
        if (!isset($this->amount)) {
            throw new MonetaSdkException(self::EXCEPTION_VAR_AMOUNT_EMPTY);
        }

        $response = [
            'id' => MonetaSdkSettings::getSettings('id', $this->configPath),
            'transactionId' => $this->transactionId,
            'amount' => number_format($this->amount, 2, '.', ''),
            'signature' => MonetaSdkService::createSignature('checkRequest', $this->transactionId, $this->configPath),
            'resultCode' => '402',
            'receipt' => $mSdkR->getJsonReceipt()
        ];

        header('Content-Type: application/json; charset=utf-8');
        echo json_encode($response, JSON_UNESCAPED_UNICODE);
        die;
    }

    /**
     * @param MonetaSdkReceipt $mSdkR
     * @param string $transactionId 
     * @return string
     * @throws MonetaSdkException
     */
    public function responseToPaymentNotification(?MonetaSdkReceipt $mSdkR = null) {
        if (isset($mSdkR)) {
            $xmlResponse  = '<?xml version="1.0" encoding="UTF-8"?>';
            $xmlResponse .= '<MNT_RESPONSE>';
            $xmlResponse .= '   <MNT_ID>' . MonetaSdkSettings::getSettings('id', $this->configPath) . '</MNT_ID>';
            $xmlResponse .= '   <MNT_TRANSACTION_ID>' . $this->transactionId . '</MNT_TRANSACTION_ID>';
            $xmlResponse .= '   <MNT_RESULT_CODE>200</MNT_RESULT_CODE>';
            $xmlResponse .= '   <MNT_SIGNATURE>' . MonetaSdkService::createSignature('paymentNotification', $this->transactionId, $this->configPath) . '</MNT_SIGNATURE>';
            $xmlResponse .= '   <MNT_ATTRIBUTES>';
            $xmlResponse .= '       <MNT_ATTRIBUTE>';
            $xmlResponse .= '           <KEY>INVENTORY</KEY>';
            $xmlResponse .= '           <VALUE>' . $mSdkR->getXmlItems() . '</VALUE>';
            $xmlResponse .= '       </MNT_ATTRIBUTE>';

            if (isset($mSdkR->customer['email'])) {
                $xmlResponse .= '       <MNT_ATTRIBUTE>';
                $xmlResponse .= '           <KEY>CUSTOMER</KEY>';
                $xmlResponse .= '           <VALUE>' . $mSdkR->customer['email'] . '</VALUE>';
                $xmlResponse .= '       </MNT_ATTRIBUTE>';
            }

            if (isset($mSdkR->customer['phone'])) {
                $xmlResponse .= '       <MNT_ATTRIBUTE>';
                $xmlResponse .= '           <KEY>PHONE</KEY>';
                $xmlResponse .= '           <VALUE>' . $mSdkR->customer['phone'] . '</VALUE>';
                $xmlResponse .= '       </MNT_ATTRIBUTE>';
            }

            if (isset($mSdkR->delivery)) {
                $xmlResponse .= '       <MNT_ATTRIBUTE>';
                $xmlResponse .= '           <KEY>DELIVERY</KEY>';
                $xmlResponse .= '           <VALUE>' . number_format($mSdkR->delivery, 2, '.', '') . '</VALUE>';
                $xmlResponse .= '       </MNT_ATTRIBUTE>';
            }

            $xmlResponse .= '   </MNT_ATTRIBUTES>';
            $xmlResponse .= '</MNT_RESPONSE>';

            header("Content-type: application/xml");
            echo $xmlResponse;
            die;
        }

        echo 'SUCCESS';
        die;
    }

    /**
     * @param string $operationId
     * @return string
     */
    public function getPaymentLink($operationId) {
        $endpoint = MonetaSdkSettings::getSettings('demo_mode', $this->configPath) ? self::DEMO_ENDPOINT : self::PROD_ENDPOINT;

        $httpParams = [
            'operationId' => $operationId,
            'moneta.locale' => MonetaSdkSettings::getSettings('locale', $this->configPath),
            'paymentSystem.unitId' => MonetaSdkSettings::getSettings('unit_id', $this->configPath),
            'paymentSystem.limitIds' => MonetaSdkSettings::getSettings('limit_ids', $this->configPath),
            'MNT_SUCCESS_URL' => MonetaSdkSettings::getSettings('success', $this->configPath),
            'MNT_INPROGRESS_URL' => MonetaSdkSettings::getSettings('inprogress', $this->configPath),
            'MNT_FAIL_URL' => MonetaSdkSettings::getSettings('fail', $this->configPath),
            'MNT_RETURN_URL' => MonetaSdkSettings::getSettings('return', $this->configPath),
            'MNT_SIGNATURE' => MonetaSdkService::createSignature('generateLink', $this->transactionId, $this->configPath, number_format($this->amount, 2, '.', ''),),
        ];

        $httpParams = array_diff($httpParams, ['']);

        $paymentLink = $endpoint . http_build_query($httpParams);

        return $paymentLink;
    }

    /** 
     * @return int
     * @throws MonetaSdkException
     */
    public function createInvoice($isTwoPhase = false) {
        if (!isset($this->amount)) {
            throw new MonetaSdkException(self::EXCEPTION_VAR_AMOUNT_EMPTY);
        }

        $params = [
            'Envelope' => [
                'Header' => [
                    'Security' => [
                        'UsernameToken' => [
                            'Username' => MonetaSdkSettings::getSettings('username', $this->configPath),
                            'Password' => MonetaSdkSettings::getSettings('password', $this->configPath)
                        ]
                    ]
                ],
                'Body' => [
                    'InvoiceRequest' => [
                        'payer' => 'card',
                        'payee' => MonetaSdkSettings::getSettings('id', $this->configPath),
                        'amount' => number_format($this->amount, 2, '.', ''),
                        'clientTransaction' => (string) $this->transactionId,

                    ]
                ]
            ]
        ];

        if ($isTwoPhase) {
            $params['Envelope']['Body']['InvoiceRequest']['operationInfo']['attribute'] = [
                [
                    'key' => 'AUTHORIZEONLY',
                    'value' => '1'
                ]
            ];
        }

        $response = MonetaSdkService::sendCurl($params, $this->configPath);

        if (!isset($response['Envelope']['Body']['InvoiceResponse']['transaction'])) {
            throw new MonetaSdkException(self::EXCEPTION_MERCHANT_API_NO_TRANSACTION_ID . " Request result: " . json_encode($response, JSON_UNESCAPED_UNICODE));
        }

        return $response['Envelope']['Body']['InvoiceResponse']['transaction'];
    }

    /**
     * @param string $operationId
     * @return array
     */
    public function confirmInvoice($operationId) {
        $params = [
            'Envelope' => [
                'Header' => [
                    'Security' => [
                        'UsernameToken' => [
                            'Username' => MonetaSdkSettings::getSettings('username', $this->configPath),
                            'Password' => MonetaSdkSettings::getSettings('password', $this->configPath)
                        ]
                    ]
                ],
                'Body' => [
                    'ConfirmTransactionRequest' => [
                        'transactionId' => $operationId,
                        'paymentPassword' => MonetaSdkSettings::getSettings('payment_password', $this->configPath),
                    ]
                ]
            ]
        ];

        $response = MonetaSdkService::sendCurl($params, $this->configPath);

        return $response['Envelope']['Body'];
    }

    /**
     * @param string $operationId
     * @return array
     */
    public function cancelInvoice($operationId) {
        $params = [
            'Envelope' => [
                'Header' => [
                    'Security' => [
                        'UsernameToken' => [
                            'Username' => MonetaSdkSettings::getSettings('username', $this->configPath),
                            'Password' => MonetaSdkSettings::getSettings('password', $this->configPath)
                        ]
                    ]
                ],
                'Body' => [
                    'CancelTransactionRequest' => [
                        'transactionId' => $operationId
                    ]
                ]
            ]
        ];

        $response = MonetaSdkService::sendCurl($params, $this->configPath);

        return $response['Envelope']['Body'];
    }

    /** 
     * @return int
     * @throws MonetaSdkException
     */
    public function createRecurrentInvoice() {
        if (!isset($this->amount)) {
            throw new MonetaSdkException(self::EXCEPTION_VAR_AMOUNT_EMPTY);
        }

        $params = [
            'Envelope' => [
                'Header' => [
                    'Security' => [
                        'UsernameToken' => [
                            'Username' => MonetaSdkSettings::getSettings('username', $this->configPath),
                            'Password' => MonetaSdkSettings::getSettings('password', $this->configPath)
                        ]
                    ]
                ],
                'Body' => [
                    'InvoiceRequest' => [
                        'payer' => 'card',
                        'payee' => MonetaSdkSettings::getSettings('id', $this->configPath),
                        'amount' => number_format($this->amount, 2, '.', ''),
                        'clientTransaction' => (string) $this->transactionId,
                        'description' => $this->description ?? 'Рекуррентный платеж по заказу №' . $this->transactionId,
                        'operationInfo' => [
                            'attribute' => [
                                [
                                    'key' => 'PAYMENTTOKEN',
                                    'value' => 'request'
                                ]
                            ]
                        ]
                    ]
                ]
            ]
        ];

        $response = MonetaSdkService::sendCurl($params, $this->configPath);

        if (!isset($response['Envelope']['Body']['InvoiceResponse']['transaction'])) {
            throw new MonetaSdkException(self::EXCEPTION_MERCHANT_API_NO_TRANSACTION_ID . " Request result: " . json_encode($response, JSON_UNESCAPED_UNICODE));
        }

        return $response['Envelope']['Body']['InvoiceResponse']['transaction'];
    }

    /**
     * @param string $token 
     * @return array
     */
    public function recurringPayment($token) {
        if (!isset($this->amount)) {
            throw new MonetaSdkException(self::EXCEPTION_VAR_AMOUNT_EMPTY);
        }

        $params = [
            'Envelope' => [
                'Header' => [
                    'Security' => [
                        'UsernameToken' => [
                            'Username' => MonetaSdkSettings::getSettings('username', $this->configPath),
                            'Password' => MonetaSdkSettings::getSettings('password', $this->configPath)
                        ]
                    ]
                ],
                'Body' => [
                    'PaymentRequest' => [
                        'payer' => 'card',
                        'payee' => MonetaSdkSettings::getSettings('id', $this->configPath),
                        'amount' => number_format($this->amount, 2, '.', ''),
                        'clientTransaction' => (string) $this->transactionId . '_MONETA' . time(),
                        'description' => $this->description ?? 'Рекуррентный платеж по заказу ' . $this->transactionId,
                        'operationInfo' => [
                            'attribute' => [
                                [
                                    'key' => 'PAYMENTTOKEN',
                                    'value' => $token
                                ]
                            ]
                        ]
                    ]
                ]
            ]
        ];

        $response = MonetaSdkService::sendCurl($params, $this->configPath);

        return $response['Envelope']['Body'];
    }

    /**
     * @param int $operationId 
     * @return array
     */
    public function refund($operationId) {
        if (!isset($this->amount)) {
            throw new MonetaSdkException(self::EXCEPTION_VAR_AMOUNT_EMPTY);
        }

        $params = [
            'Envelope' => [
                'Header' => [
                    'Security' => [
                        'UsernameToken' => [
                            'Username' => MonetaSdkSettings::getSettings('username', $this->configPath),
                            'Password' => MonetaSdkSettings::getSettings('password', $this->configPath)
                        ]
                    ]
                ],
                'Body' => [
                    'RefundRequest' => [
                        'amount' => number_format($this->amount, 2, '.', ''),
                        'paymentPassword' => MonetaSdkSettings::getSettings('payment_password', $this->configPath),
                        'transactionId' => $operationId,
                        'description' => $this->description ?? 'Возврат по операции ' . $this->transactionId,
                        'clientTransaction' => (string) $this->transactionId,
                    ]
                ]
            ]
        ];

        $response = MonetaSdkService::sendCurl($params, $this->configPath);

        return $response['Envelope']['Body'];
    }

    /**
     * @param string $operationId 
     * @return array
     */
    public function getOperationDetailsById($operationId) {
        $params = [
            'Envelope' => [
                'Header' => [
                    'Security' => [
                        'UsernameToken' => [
                            'Username' => MonetaSdkSettings::getSettings('username', $this->configPath),
                            'Password' => MonetaSdkSettings::getSettings('password', $this->configPath)
                        ]
                    ]
                ],
                'Body' => [
                    'GetOperationDetailsByIdRequest' => $operationId
                ]
            ]
        ];

        $response = MonetaSdkService::sendCurl($params, $this->configPath);

        return $response['Envelope']['Body'];
    }
}
